@@ -1,5 +1,5 @@
 # engine/default.py
-# Copyright (C) 2005-2019 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2020 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
@@ -281,6 +281,13 @@ class DefaultDialect(interfaces.Dialect):
 
     @property
     def supports_sane_rowcount_returning(self):
+        """True if this dialect supports sane rowcount even if RETURNING is
+        in use.
+
+        For dialects that don't support RETURNING, this is synomous
+        with supports_sane_rowcount.
+
+        """
         return self.supports_sane_rowcount
 
     @classmethod
@@ -783,6 +790,8 @@ class DefaultExecutionContext(interfaces.ExecutionContext):
         processors = compiled._bind_processors
 
         if compiled.contains_expanding_parameters:
+            # copy processors for this case as they will be mutated
+            processors = dict(processors)
             positiontup = self._expand_in_parameters(compiled, processors)
         elif compiled.positional:
             positiontup = self.compiled.positiontup
