@@ -1,5 +1,5 @@
 # sql/base.py
-# Copyright (C) 2005-2016 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2020 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
@@ -10,14 +10,16 @@
 """
 
 
-from .. import util, exc
 import itertools
-from .visitors import ClauseVisitor
 import re
-import collections
 
-PARSE_AUTOCOMMIT = util.symbol('PARSE_AUTOCOMMIT')
-NO_ARG = util.symbol('NO_ARG')
+from .visitors import ClauseVisitor
+from .. import exc
+from .. import util
+
+
+PARSE_AUTOCOMMIT = util.symbol("PARSE_AUTOCOMMIT")
+NO_ARG = util.symbol("NO_ARG")
 
 
 class Immutable(object):
@@ -46,7 +48,7 @@ def _generative(fn, *args, **kw):
     return self
 
 
-class _DialectArgView(collections.MutableMapping):
+class _DialectArgView(util.collections_abc.MutableMapping):
     """A dictionary view of dialect-level arguments in the form
     <dialectname>_<argument_name>.
 
@@ -78,7 +80,8 @@ class _DialectArgView(collections.MutableMapping):
             dialect, value_key = self._key(key)
         except KeyError:
             raise exc.ArgumentError(
-                "Keys must be of the form <dialectname>_<argname>")
+                "Keys must be of the form <dialectname>_<argname>"
+            )
         else:
             self.obj.dialect_options[dialect][value_key] = value
 
@@ -87,19 +90,22 @@ class _DialectArgView(collections.MutableMapping):
         del self.obj.dialect_options[dialect][value_key]
 
     def __len__(self):
-        return sum(len(args._non_defaults) for args in
-                   self.obj.dialect_options.values())
+        return sum(
+            len(args._non_defaults)
+            for args in self.obj.dialect_options.values()
+        )
 
     def __iter__(self):
         return (
             util.safe_kwarg("%s_%s" % (dialect_name, value_name))
             for dialect_name in self.obj.dialect_options
-            for value_name in
-            self.obj.dialect_options[dialect_name]._non_defaults
+            for value_name in self.obj.dialect_options[
+                dialect_name
+            ]._non_defaults
         )
 
 
-class _DialectArgDict(collections.MutableMapping):
+class _DialectArgDict(util.collections_abc.MutableMapping):
     """A dictionary view of dialect-level arguments for a specific
     dialect.
 
@@ -188,8 +194,8 @@ class DialectKWArgs(object):
         if construct_arg_dictionary is None:
             raise exc.ArgumentError(
                 "Dialect '%s' does have keyword-argument "
-                "validation and defaults enabled configured" %
-                dialect_name)
+                "validation and defaults enabled configured" % dialect_name
+            )
         if cls not in construct_arg_dictionary:
             construct_arg_dictionary[cls] = {}
         construct_arg_dictionary[cls][argument_name] = default
@@ -231,6 +237,7 @@ class DialectKWArgs(object):
         if dialect_cls.construct_arguments is None:
             return None
         return dict(dialect_cls.construct_arguments)
+
     _kw_registry = util.PopulateDict(_kw_reg_for_dialect)
 
     def _kw_reg_for_dialect_cls(self, dialect_name):
@@ -275,11 +282,12 @@ class DialectKWArgs(object):
             return
 
         for k in kwargs:
-            m = re.match('^(.+?)_(.+)$', k)
+            m = re.match("^(.+?)_(.+)$", k)
             if not m:
                 raise TypeError(
                     "Additional arguments should be "
-                    "named <dialectname>_<argument>, got '%s'" % k)
+                    "named <dialectname>_<argument>, got '%s'" % k
+                )
             dialect_name, arg_name = m.group(1, 2)
 
             try:
@@ -287,20 +295,22 @@ class DialectKWArgs(object):
             except exc.NoSuchModuleError:
                 util.warn(
                     "Can't validate argument %r; can't "
-                    "locate any SQLAlchemy dialect named %r" %
-                    (k, dialect_name))
+                    "locate any SQLAlchemy dialect named %r"
+                    % (k, dialect_name)
+                )
                 self.dialect_options[dialect_name] = d = _DialectArgDict()
                 d._defaults.update({"*": None})
                 d._non_defaults[arg_name] = kwargs[k]
             else:
-                if "*" not in construct_arg_dictionary and \
-                        arg_name not in construct_arg_dictionary:
+                if (
+                    "*" not in construct_arg_dictionary
+                    and arg_name not in construct_arg_dictionary
+                ):
                     raise exc.ArgumentError(
                         "Argument %r is not accepted by "
-                        "dialect %r on behalf of %r" % (
-                            k,
-                            dialect_name, self.__class__
-                        ))
+                        "dialect %r on behalf of %r"
+                        % (k, dialect_name, self.__class__)
+                    )
                 else:
                     construct_arg_dictionary[arg_name] = kwargs[k]
 
@@ -355,33 +365,48 @@ class Executable(Generative):
 
         .. seealso::
 
-            :meth:`.Connection.execution_options()`
+            :meth:`.Connection.execution_options`
 
-            :meth:`.Query.execution_options()`
+            :meth:`.Query.execution_options`
+
+            :meth:`.Executable.get_execution_options`
 
         """
-        if 'isolation_level' in kw:
+        if "isolation_level" in kw:
             raise exc.ArgumentError(
                 "'isolation_level' execution option may only be specified "
                 "on Connection.execution_options(), or "
                 "per-engine using the isolation_level "
                 "argument to create_engine()."
             )
-        if 'compiled_cache' in kw:
+        if "compiled_cache" in kw:
             raise exc.ArgumentError(
                 "'compiled_cache' execution option may only be specified "
                 "on Connection.execution_options(), not per statement."
             )
         self._execution_options = self._execution_options.union(kw)
 
+    def get_execution_options(self):
+        """ Get the non-SQL options which will take effect during execution.
+
+        .. versionadded:: 1.3
+
+        .. seealso::
+
+            :meth:`.Executable.execution_options`
+        """
+        return self._execution_options
+
     def execute(self, *multiparams, **params):
         """Compile and execute this :class:`.Executable`."""
         e = self.bind
         if e is None:
-            label = getattr(self, 'description', self.__class__.__name__)
-            msg = ('This %s is not directly bound to a Connection or Engine.'
-                   'Use the .execute() method of a Connection or Engine '
-                   'to execute this construct.' % label)
+            label = getattr(self, "description", self.__class__.__name__)
+            msg = (
+                "This %s is not directly bound to a Connection or Engine. "
+                "Use the .execute() method of a Connection or Engine "
+                "to execute this construct." % label
+            )
             raise exc.UnboundExecutionError(msg)
         return e._execute_clauseelement(self, multiparams, params)
 
@@ -435,7 +460,7 @@ class SchemaEventTarget(object):
 class SchemaVisitor(ClauseVisitor):
     """Define the visiting for ``SchemaItem`` objects."""
 
-    __traverse_options__ = {'schema_visitor': True}
+    __traverse_options__ = {"schema_visitor": True}
 
 
 class ColumnCollection(util.OrderedProperties):
@@ -447,11 +472,11 @@ class ColumnCollection(util.OrderedProperties):
 
     """
 
-    __slots__ = '_all_columns'
+    __slots__ = "_all_columns"
 
     def __init__(self, *columns):
         super(ColumnCollection, self).__init__()
-        object.__setattr__(self, '_all_columns', [])
+        object.__setattr__(self, "_all_columns", [])
         for c in columns:
             self.add(c)
 
@@ -486,8 +511,9 @@ class ColumnCollection(util.OrderedProperties):
 
         self._data[column.key] = column
         if remove_col is not None:
-            self._all_columns[:] = [column if c is remove_col
-                                    else c for c in self._all_columns]
+            self._all_columns[:] = [
+                column if c is remove_col else c for c in self._all_columns
+            ]
         else:
             self._all_columns.append(column)
 
@@ -500,13 +526,14 @@ class ColumnCollection(util.OrderedProperties):
         """
         if not column.key:
             raise exc.ArgumentError(
-                "Can't add unnamed column to column collection")
+                "Can't add unnamed column to column collection"
+            )
         self[column.key] = column
 
     def __delitem__(self, key):
         raise NotImplementedError()
 
-    def __setattr__(self, key, object):
+    def __setattr__(self, key, obj):
         raise NotImplementedError()
 
     def __setitem__(self, key, value):
@@ -517,11 +544,17 @@ class ColumnCollection(util.OrderedProperties):
             # columns collection
 
             existing = self[key]
+
+            if existing is value:
+                return
+
             if not existing.shares_lineage(value):
-                util.warn('Column %r on table %r being replaced by '
-                          '%r, which has the same key.  Consider '
-                          'use_labels for select() statements.' %
-                          (key, getattr(existing, 'table', None), value))
+                util.warn(
+                    "Column %r on table %r being replaced by "
+                    "%r, which has the same key.  Consider "
+                    "use_labels for select() statements."
+                    % (key, getattr(existing, "table", None), value)
+                )
 
             # pop out memoized proxy_set as this
             # operation may very well be occurring
@@ -537,17 +570,19 @@ class ColumnCollection(util.OrderedProperties):
     def remove(self, column):
         del self._data[column.key]
         self._all_columns[:] = [
-            c for c in self._all_columns if c is not column]
+            c for c in self._all_columns if c is not column
+        ]
 
-    def update(self, iter):
-        cols = list(iter)
+    def update(self, iter_):
+        cols = list(iter_)
         all_col_set = set(self._all_columns)
         self._all_columns.extend(
-            c for label, c in cols if c not in all_col_set)
+            c for label, c in cols if c not in all_col_set
+        )
         self._data.update((label, c) for label, c in cols)
 
-    def extend(self, iter):
-        cols = list(iter)
+    def extend(self, iter_):
+        cols = list(iter_)
         all_col_set = set(self._all_columns)
         self._all_columns.extend(c for c in cols if c not in all_col_set)
         self._data.update((c.key, c) for c in cols)
@@ -569,12 +604,11 @@ class ColumnCollection(util.OrderedProperties):
         return util.OrderedProperties.__contains__(self, other)
 
     def __getstate__(self):
-        return {'_data': self._data,
-                '_all_columns': self._all_columns}
+        return {"_data": self._data, "_all_columns": self._all_columns}
 
     def __setstate__(self, state):
-        object.__setattr__(self, '_data', state['_data'])
-        object.__setattr__(self, '_all_columns', state['_all_columns'])
+        object.__setattr__(self, "_data", state["_data"])
+        object.__setattr__(self, "_all_columns", state["_all_columns"])
 
     def contains_column(self, col):
         return col in set(self._all_columns)
@@ -586,7 +620,7 @@ class ColumnCollection(util.OrderedProperties):
 class ImmutableColumnCollection(util.ImmutableProperties, ColumnCollection):
     def __init__(self, data, all_columns):
         util.ImmutableProperties.__init__(self, data)
-        object.__setattr__(self, '_all_columns', all_columns)
+        object.__setattr__(self, "_all_columns", all_columns)
 
     extend = remove = util.ImmutableProperties._immutable
 
@@ -619,15 +653,18 @@ def _bind_or_error(schemaitem, msg=None):
     bind = schemaitem.bind
     if not bind:
         name = schemaitem.__class__.__name__
-        label = getattr(schemaitem, 'fullname',
-                        getattr(schemaitem, 'name', None))
+        label = getattr(
+            schemaitem, "fullname", getattr(schemaitem, "name", None)
+        )
         if label:
-            item = '%s object %r' % (name, label)
+            item = "%s object %r" % (name, label)
         else:
-            item = '%s object' % name
+            item = "%s object" % name
         if msg is None:
-            msg = "%s is not bound to an Engine or Connection.  "\
-                "Execution can not proceed without a database to execute "\
+            msg = (
+                "%s is not bound to an Engine or Connection.  "
+                "Execution can not proceed without a database to execute "
                 "against." % item
+            )
         raise exc.UnboundExecutionError(msg)
     return bind
